/*
 * arch/arch/mach-ixp4xx/ng-pci.c
 *
 * PCI setup routines for Netgear FVX538/FVS338/DGFV338
 *
 * Copyright (c) 2006, TeamF1, Inc.
 * Copyright (C) 2002 Jungo Software Technologies.
 * Copyright (C) 2003 MontaVista Softwrae, Inc.
 *
 * Maintainer: Deepak Saxena <dsaxena@mvista.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */

#include <linux/pci.h>
#include <linux/init.h>
#include <linux/delay.h>

#include <asm/mach-types.h>
#include <asm/hardware.h>
#include <asm/irq.h>

#include <asm/mach/pci.h>

/* defines */
#define PCI_CLK_TOGGLE  0x100
#define PCI_CLK_DISABLE 0x1
#define PCI_CLK_ENABLE  0x2
#define PCI_CLK_33MHZ   0x1

extern void ixp4xx_pci_preinit(void);
extern int ixp4xx_setup(int nr, struct pci_sys_data *sys);
extern struct pci_bus *ixp4xx_scan_bus(int nr, struct pci_sys_data *sys);

void dgfv_pci_clk_toggle (int toggle)
{
    volatile unsigned int gpioClkReg;

    gpioClkReg = *(volatile unsigned int *)(IXP4XX_GPIO_GPCLKR);

    switch (toggle)
        {
        case PCI_CLK_DISABLE:
            gpioClkReg &= ~PCI_CLK_TOGGLE;
            break;
        case PCI_CLK_ENABLE:
            gpioClkReg |= PCI_CLK_TOGGLE;
            break;
        }

    *(volatile unsigned int *)(IXP4XX_GPIO_GPCLKR) = gpioClkReg;
}

void dgfv_pci_clk_config (void)
{
    volatile unsigned int gpioClkReg;
    unsigned int dutyCycle;

    gpioClkReg = *(volatile unsigned int *)(IXP4XX_GPIO_GPCLKR);

    /* configure clock 0 for pci */

    gpioClkReg |= (PCI_CLK_33MHZ << 4);
    dutyCycle = (unsigned int)(PCI_CLK_33MHZ/2);
    gpioClkReg |= (dutyCycle << 0);

    *(volatile unsigned int *)(IXP4XX_GPIO_GPCLKR) = gpioClkReg;
}

void __init ng_pci_preinit(void)
{
#ifdef CONFIG_ARCH_NG_DGFV338
    int i;

    /* 
     * these changes are added for dgfv338 platform as
     * software workaround for pci reset h/w issue, 
     * in warm reboot
     */

    dgfv_pci_clk_toggle(PCI_CLK_DISABLE);

    gpio_line_config (NG_DGFV338_PCI_CLK, IXP4XX_GPIO_OUT);

    gpio_line_config(NG_DGFV338_PCI_SLOT0_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_config(NG_DGFV338_PCI_SLOT1_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_config(NG_DGFV338_PCI_SLOT2_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_isr_clear(NG_DGFV338_PCI_SLOT0_PIN);
    gpio_line_isr_clear(NG_DGFV338_PCI_SLOT1_PIN);
    gpio_line_isr_clear(NG_DGFV338_PCI_SLOT2_PIN);

    
    for (i=0; i < 50; i++)
        {
        udelay(1000);
        }

    dgfv_pci_clk_config();
    dgfv_pci_clk_toggle(PCI_CLK_ENABLE);

    for (i=0; i < 10; i++)
        {
        udelay(1000);
        }

    gpio_line_set (NG_DGFV338_SW_RESET, IXP4XX_GPIO_HIGH);

    for (i=0; i < 5; i++)
        {
        udelay(1000);
        }

#else
    gpio_line_config(NG_PCI_SLOT0_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_config(NG_PCI_SLOT1_PIN,
                     IXP4XX_GPIO_IN | IXP4XX_GPIO_ACTIVE_LOW);

    gpio_line_isr_clear(NG_PCI_SLOT0_PIN);
    gpio_line_isr_clear(NG_PCI_SLOT1_PIN);
#endif

    ixp4xx_pci_preinit();
}

static int __init ng_map_irq(struct pci_dev *dev, u8 slot, u8 pin)
{
#ifdef CONFIG_ARCH_NG_DGFV338
    if (slot == NG_PCI_SLOT0_DEVID)
        return IRQ_NG_DGFV338_PCI_INTA;
#else
    if (slot == NG_PCI_SLOT0_DEVID)
        return IRQ_NG_PCI_SLOT0;
#endif
    else if (slot == NG_PCI_SLOT1_DEVID)
        return IRQ_NG_PCI_SLOT1;
    return -1;
}

struct hw_pci ng_pci __initdata = {
    .nr_controllers = 1,
    .preinit =        ng_pci_preinit,
    .swizzle =        pci_std_swizzle,
    .setup =          ixp4xx_setup,
    .scan =           ixp4xx_scan_bus,
    .map_irq =        ng_map_irq,
};

int __init ng_pci_init(void)
{
    if (machine_is_ng_fvx538() || machine_is_ng_dgfv338())
        pci_common_init(&ng_pci);
    return 0;
}

subsys_initcall(ng_pci_init);
